/*
 *  forecast - query weather forecasts from forecast.io
 *  Copyright (C) 2015 Jens John <dev@2ion.de>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RENDER_H
#define RENDER_H

#include <assert.h>
#include <json-c/json.h>
#include <json-c/json_object.h>
#include <string.h>
#include <time.h>

#include "forecast.h"

#define RENDER_BEARING(deg) \
  deg ==  0.00 ? "N" : (deg <  45.0 ? "NNE" : (deg ==  45.0 ? "NE" : (deg <  90.0 ? "ENE" : \
( deg ==  90.0 ? "E" : (deg < 135.0 ? "ESE" : (deg == 135.0 ? "SE" : (deg < 180.0 ? "SSE" : \
( deg == 180.0 ? "S" : (deg < 225.0 ? "SSW" : (deg == 225.0 ? "SW" : (deg < 270.0 ? "WSW" : \
( deg == 270.0 ? "W" : (deg < 315.0 ? "WNW" : (deg == 315.0 ? "NW" : "NNW"))))))))))))))

#define _PASTE(x, y) x ## _ ## y

#define PASTE(x, y) _PASTE(x, y)

#define NAME(prefix, name) PASTE(prefix, name)

#define EXTRACT(object, key)  \
  struct json_object *(key);  \
  json_object_object_get_ex((object), #key, &(key));

#define EXTRACT_PREFIXED(object, key)                                                         \
  struct json_object *(NAME(object, key)) = NULL;                                             \
  if(json_object_object_get_ex((object), #key, &(NAME(object, key))) != TRUE)                 \
    LERROR(EXIT_FAILURE, 0, "json_object_object_get_ex() failed: %s", #key);

char*   render_time(struct json_object*);
double  render_f2c(double fahrenheit);
double  render_mph2kph(double mph);
int     render(const Config *c, Data *d);
int     render_datapoint(struct json_object *d);
void    render_hourly_datapoints(const PlotCfg*, struct json_object*);
void    render_hourly_datapoints_plot(const PlotCfg*, struct json_object*);
void    render_daily_temperature_plot(const PlotCfg*, struct json_object*);
void    render_precipitation_plot_daily(const PlotCfg *, struct json_object*);
void    render_precipitation_plot_hourly(const PlotCfg *, struct json_object*);
void    render_daylight(const PlotCfg*, struct json_object*);

#endif
